import { Component, HostListener } from "@angular/core";
import { Router } from "@angular/router";
import { BuildingService } from "../../service/monitoring-api.service";
import { DeviceService } from "../../service/device.service";
import { ToastrService } from "ngx-toastr";
import { LoginService } from "../../service/login.service";
import { NgbModal } from "@ng-bootstrap/ng-bootstrap";
import { ControlSchedulerComponent } from "../control-scheduler/control-scheduler.component";

@Component({
  selector: "app-device-control",
  templateUrl: "./device-control.component.html",
  styleUrls: ["./device-control.component.css"],
})
export class DeviceControlComponent {
  data: any;

  filteredRows: any[];
  filteredDeviceRows: any[];
  searchTerm: string = "";
  buildingSelected: any;
  statusSelected: any;
  categorySelected: any;
  dataMasterCategori: any;
  dataBuildingList: any;
  dataMasterStatus: any;
  storedData: any;
  spinnerFilterActive = false;
  switchState: boolean;
  public breadcrumb: any;
  spinnerActive: boolean = false;
  singlebasicSelected: any;

  constructor(
    private router: Router,
    private deviceService: DeviceService,
    private monitoringApiService: BuildingService,
    private toastr: ToastrService,
    private authService: LoginService,
    private modalService: NgbModal,
  ) {}

  ngOnInit() {
    this.authService.startTokenCheck();
    this.authService.startTrackingActivity();
    this.breadcrumb = {
      mainlabel: "Device",
      links: [
        {
          name: "Home",
          isLink: false,
          link: "/dashboard/sales",
        },
        {
          name: "Device",
          isLink: false,
        },
      ],
    };
    this.storedData = JSON.parse(localStorage.getItem("currentUser"));
    this.buildingSelected = this.storedData.buildingId;
    this.fetchData(
      this.buildingSelected,
      this.categorySelected,
      this.statusSelected
    );
    this.dataListMaster();
    this.dataListBuilding();
  }

  fetchData(buildingSelected, categorySelected, statusSelected) {
    this.deviceService
      .getDeviceData(buildingSelected, categorySelected, statusSelected)
      .subscribe((res) => {
        this.data = res;
        this.filteredRows = this.data.results.data;
        this.filteredDeviceRows = this.filterDevices(this.filteredRows);
      });
  }

  filterDevices(devices: any[]): any[] {
    return devices.filter((device) =>
      device.mapping.some(
        (map) => map.code.startsWith("switch") && map.type === "Boolean"
      )
    );
  }

  dataListMaster() {
    this.monitoringApiService.getMasterListData().subscribe((data) => {
      const dataCategory = data.data.find(
        (item) => item.name === "master_category"
      ).headerDetailParam;
      const dataStatus = data.data.find(
        (item) => item.name === "master_status"
      ).headerDetailParam;
      this.dataMasterCategori = dataCategory.filter(
        (item) => item.statusName.toLowerCase() === "aktif"
      );
      this.dataMasterStatus = dataStatus.filter((item) => item.statusName.toLowerCase() === "aktif");
    });
  }

  dataListBuilding() {
    this.monitoringApiService.getBuildingList().subscribe((data) => {
      this.dataBuildingList = data.data.filter((item) => item.statusName.toLowerCase() === "aktif");
    });
  }

  doFilter() {
    if (!this.buildingSelected) {
      this.toastr.error("Warning", "Filter Building tidak boleh kosong.", {
        timeOut: 5000,
        closeButton: true,
      });
    } else {
      this.spinnerFilterActive = true;
      this.fetchData(
        this.buildingSelected,
        this.categorySelected,
        this.statusSelected
      );
      setTimeout(() => {
        this.spinnerFilterActive = false;
      }, 3000);
    }
  }

  doCancelFilter() {
    this.storedData = JSON.parse(localStorage.getItem("currentUser"));
    this.buildingSelected = this.storedData.buildingId;
    this.categorySelected = undefined;
    this.statusSelected = undefined;
    this.fetchData(this.buildingSelected, 0, 0);
  }

  filterRows() {
    if (!this.searchTerm) {
      this.filteredRows = [...this.data.results.data];
    } else {
      this.filteredRows = this.data.results.data.filter((row) =>
        this.rowContainsSearchTerm(row)
      );
    }
  }

  rowContainsSearchTerm(row: any): boolean {
    const searchTermLC = this.searchTerm.toLowerCase();
    return Object.values(row).some(
      (value) =>
        value !== null && value.toString().toLowerCase().includes(searchTermLC)
    );
  }

  switchChanged(ev, data) {
    const requestData = {
      id: data.id,
      code: data.mapping[0].code,
      value: ev,
      command_type: "on_off",
    };
    this.deviceService.deviceSwitch(requestData).subscribe((res) => {
      console.log(res);
    });
  }

  toggleMenu(itemId) {
    this.filteredDeviceRows.forEach(item => {
      if (item.id === itemId) {
        item.menuOpen = !item.menuOpen;
      } else {
        item.menuOpen = false;  // Close other menus
      }
    });
  }

  schedulerItem(row) {
    console.log(row);
    
    this.router.navigate(["/device/scheduler", row.id]);
  }

  // schedulerItem(item) {
  //   const modalRef = this.modalService.open(ControlSchedulerComponent, {
  //     size: "md",
  //     backdrop: "static", 
  //     keyboard: false, 
  //     centered: true
  //   });

  //   modalRef.componentInstance.device = item;
  //   modalRef.result.then(
  //     (result) => {
  //       if (result) {
  //         console.log(result);
  //       }
  //     },
  //     (reason) => {
  //       console.log(`Dismissed: ${reason}`);
  //     }
  //   );
  //   // Handle edit action
  // }

  @HostListener('document:click', ['$event'])
  clickout(event) {
    if (!event.target.closest('.menu-popup') && !event.target.closest('.btn')) {
      this.filteredDeviceRows.forEach(item => {
        item.menuOpen = false;
      });
    }
  }
}
