import { Component, Input, OnInit } from "@angular/core";
import { FormBuilder, FormGroup, Validators } from "@angular/forms";
import { NgbActiveModal } from "@ng-bootstrap/ng-bootstrap";
import { BuildingService } from "../../service/monitoring-api.service";
import { LoginService } from "../../service/login.service";

@Component({
  selector: "app-add-edit-master",
  templateUrl: "./add-edit-master.component.html",
  styleUrls: ["./add-edit-master.component.css"],
})
export class AddEditMasterComponent implements OnInit {
  @Input() headerId: number;
  @Input() dataRow: any;
  @Input() mode: any;
  @Input() category: boolean = false;
  @Input() floor: boolean = false;
  myForm: FormGroup;
  dataMasterStatus: any;
  selectedIcon: string = '';
  labelModal: string = '';

  constructor(
    public activeModal: NgbActiveModal,
    private fb: FormBuilder,
    private monitoringApiService: BuildingService,
    private authService: LoginService
  ) {}

  ngOnInit() {
    this.authService.startTokenCheck();
    this.authService.startTrackingActivity();
    this.createForm();
    this.dataListMaster();
    if (this.mode === "add") {
      this.labelModal = "Add Row"
    } else {
      this.labelModal = "Edit Row"
    }
    
    if (this.dataRow) {
      this.editForm();
    }
  }

  createForm() {
    const formControls = {
      name: ["", Validators.required],
      status: ["", Validators.required],
      headerId: this.headerId
    };

    if (this.category) {
      formControls['icon'] = [undefined, Validators.required];
    }

    if (this.floor) {
      formControls['code'] = ['', Validators.required];
    }

    this.myForm = this.fb.group(formControls);
  }

  toUppercase(event: Event) {
    const input = event.target as HTMLInputElement;
    input.value = input.value.toUpperCase();
    this.myForm.get('code').setValue(input.value);
  }


  editForm() {
    const formControls = {
      id: this.dataRow.id,
      name: [this.dataRow.name, Validators.required],
      status: [this.dataRow.status, Validators.required],
      headerId: this.dataRow.header_id,
    };

    if (this.category) {
      formControls['icon'] = [this.dataRow.icon || undefined , Validators.required];
      this.selectedIcon = this.dataRow.icon || undefined;
    }

    if (this.floor) {
      formControls['code'] = [this.dataRow.code || '' , Validators.required];
    }
    this.myForm = this.fb.group(formControls);
  }

  dataListMaster() {
    this.monitoringApiService.getMasterListData().subscribe((data) => {
      const dataCategory = data.data.find(
        (item) => item.name === "master_status"
      ).headerDetailParam;
      this.dataMasterStatus = dataCategory.filter(item => item.statusName.toLowerCase() === "aktif" || item.status.toLowerCase() === "71" || item.status.toLowerCase() === "71");
    });
  }

  addRow() {
    if (this.myForm.valid) {
      this.activeModal.close(this.myForm.value);
    } else {
      this.markFormGroupTouched(this.myForm)
    }
  }

  markFormGroupTouched(formGroup: FormGroup) {
    (Object as any).values(formGroup.controls).forEach((control) => {
      control.markAsTouched();
      if (control.controls) {
        this.markFormGroupTouched(control);
      }
    });
  }

  addDepartment() {
    let newDept = prompt("Enter new department name:");
    if (newDept) {
      alert(`Department ${newDept} added! (simulated)`);
    }
  }

  onIconSelected(icon: string): void {
    this.selectedIcon = icon;
    this.myForm.get('icon').setValue(icon);  // Set nilai icon ke dalam form
  }
}
