import { Component } from '@angular/core';
import { BuildingService } from '../../service/monitoring-api.service';
import { DeviceService } from '../../service/device.service';
import { ActivatedRoute, Router } from '@angular/router';
import { ToastrService } from 'ngx-toastr';
import { LoginService } from '../../service/login.service';

@Component({
  selector: 'app-control-device-seemore',
  templateUrl: './control-device-seemore.component.html',
  styleUrls: ['./control-device-seemore.component.css']
})
export class ControlDeviceSeemoreComponent {
  data: any;
  filteredRows: any[];
  filteredDeviceRows: any[];
  searchTerm: string = "";
  buildingSelected: any;
  statusSelected: any;
  categorySelected: any;
  dataMasterCategori: any;
  dataBuildingList: any;
  dataMasterStatus: any;
  spinnerFilterActive = false;
  switchState: boolean;
  buildingId: any;
  floorId: any;
  roomId: any;
  modeRoute: any;

  public breadcrumb: any;
  spinnerActive: boolean = false;

  constructor(
    private deviceService: DeviceService,
    private monitoringApiService: BuildingService,
    private route: ActivatedRoute,
    private toastr: ToastrService,
    private authService: LoginService
  ) {}

  ngOnInit() {
    this.authService.startTokenCheck();
    this.authService.startTrackingActivity();
    this.route.params.subscribe((params) => {
      this.modeRoute = params["mode"];
      this.buildingId = params["buildingId"] ? params["buildingId"] : 0;
      this.floorId = params["floorId"] ? params["floorId"] : 0;
      this.roomId = params["roomId"] ? params["roomId"] : 0;
      this.buildingSelected = parseInt(this.buildingId);
    });
    this.breadcrumb = {
      mainlabel: "Control Device",
      linkBack: this.routeBack(''),
      isLinkBack: true,
      links: [
        {
          name: "Dashboard",
          isLink: false,
          link: "",
        },
        {
          name: "Energy Monitoring",
          isLink: false,
          link: "",
        },
        {
          name: "Control Device",
          isLink: false,
          link: "",
        },
      ],
    };
    
    this.fetchData(
      this.buildingSelected,
      this.categorySelected,
      this.statusSelected
    );
    this.dataListMaster();
    this.dataListBuilding();
  }

  routeBack(route){
    if (this.modeRoute === 'building') {
      route = `/monitoring/view-detail-building/${this.buildingId}`;
    } else if (this.modeRoute === 'floor') {
      route = `/monitoring/view-detail-floor/${this.buildingId}/${this.floorId}`;
    } else if (this.modeRoute === 'room') {
      route = `/monitoring/view-detail-room/${this.buildingId}/${this.floorId}/${this.roomId}`;
    }
    return route
  }

  fetchData(buildingSelected, categorySelected, statusSelected) {
    this.deviceService
      .getDeviceData(buildingSelected, categorySelected, statusSelected)
      .subscribe((res) => {
        this.data = res;
        this.filteredRows = this.data.results.data;
        this.filteredDeviceRows = this.filterDevices(this.filteredRows);
      });
  }

  filterDevices(devices: any[]): any[] {
    return devices.filter((device) =>
      device.mapping.some(
        (map) => map.code.startsWith("switch") && map.type === "Boolean"
      )
    );
  }

  dataListMaster() {
    this.monitoringApiService.getMasterListData().subscribe((data) => {
      const dataCategory = data.data.find(
        (item) => item.name === "master_category"
      ).headerDetailParam;
      const dataStatus = data.data.find(
        (item) => item.name === "master_status"
      ).headerDetailParam;
      this.dataMasterCategori = dataCategory.filter(
        (item) => item.statusName.toLowerCase() === "aktif"
      );
      this.dataMasterStatus = dataStatus.filter((item) => item.statusName.toLowerCase() === "aktif" || item.status === "71");
    });
  }

  dataListBuilding() {
    this.monitoringApiService.getBuildingList().subscribe((data) => {
      this.dataBuildingList = data.data.filter((item) => item.statusName.toLowerCase() === "aktif" || item.status_id === 71);
    });
  }

  doFilter() {
    if (!this.buildingSelected) {
      this.toastr.error("Warning", "Filter Building tidak boleh kosong.", {
        timeOut: 5000,
        closeButton: true,
      });
    } else {
      this.spinnerFilterActive = true;
      this.fetchData(
        this.buildingSelected,
        this.categorySelected,
        this.statusSelected
      );
      setTimeout(() => {
        this.spinnerFilterActive = false;
      }, 3000);
    }
    
  }

  doFilterCancel(){
    this.buildingSelected = parseInt(this.buildingId);
    this.categorySelected = undefined;
    this.statusSelected = undefined;
    this.fetchData(this.buildingSelected, 0, 0);
  }

  filterRows() {
    if (!this.searchTerm) {
      this.filteredRows = [...this.data.results.data];
    } else {
      this.filteredRows = this.data.results.data.filter((row) =>
        this.rowContainsSearchTerm(row)
      );
    }
  }

  rowContainsSearchTerm(row: any): boolean {
    const searchTermLC = this.searchTerm.toLowerCase();
    return Object.values(row).some(
      (value) =>
        value !== null && value.toString().toLowerCase().includes(searchTermLC)
    );
  }

  switchChanged(ev, data) {
    const requestData = {
      id: data.id,
      code: data.mapping[0].code,
      value: ev,
      command_type: "on_off",
    };
    this.deviceService.deviceSwitch(requestData).subscribe((res) => {
     
    });
  }
}
